import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

public class Text {

    private ArrayList<String> text = new ArrayList<>();
    private static final String dummyText = "The standard Lorem Ipsum passage, used since the 1500s \"Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.\" Section 1.10.32 of \"de Finibus Bonorum et Malorum\", written by Cicero in 45 BC \"Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?";
    private HashMap<String, ArrayList<Integer>> wordbook = new HashMap<>();

    /**
     * Method to check if a specific Paragraph exists
     *
     * @param paragraphNumber the number of the paragraph which should be checked
     * @return returns true if the paragraph exists
     */
    private boolean paragraphExists(int paragraphNumber) {
        return ((paragraphNumber > 0) && (paragraphNumber <= this.text.size()));
    }

    /**
     * Method to add a paragraph at a specific position.
     *
     * @param paragraphNumber number of paragraph where the new text should be added
     * @param text            the Text which should be added.
     * @return returns true if the given paragraph exists and is added successfully
     */
    public boolean add(int paragraphNumber, String text) {
        if (paragraphExists(paragraphNumber)) {
            this.text.add((paragraphNumber - 1), text);
            return true;
        }
        return false;
    }

    /**
     * Method to add a paragraph at the end of the existing text.
     *
     * @param text the Text which should be added.
     * @return returns true if the paragraph is added successfully
     */
    public boolean add(String text) {
        this.text.add(text);
        return true;
    }

    public ArrayList<String> getText() {
        return text;
    }

    /**
     * Method to add a dummy text paragraph at a specific position.
     *
     * @param paragraphNumber number of paragraph where the dummy text should be added
     * @return returns true if the given paragraph exists and is added successfully
     */
    public boolean dummy(int paragraphNumber) {
        return add(paragraphNumber, dummyText);
    }

    /**
     * Method to add a dummy text paragraph at the end of the existing text.
     *
     * @return returns true if the paragraph is added successfully
     */
    public boolean dummy() {
        return add(dummyText);
    }

    /**
     * Method to replace characters in a specific paragraph
     *
     * @param paragraphNumber number of paragraph which should be changed.
     * @param oldChar         the old character.
     * @param newChar         the new character.
     * @return returns true if the given paragraph exists and is changed successfully
     */
    public boolean replace(int paragraphNumber, String oldChar, String newChar) {
        if (paragraphExists(paragraphNumber)) {
            text.set((paragraphNumber - 1), text.get(paragraphNumber - 1).replace(oldChar, newChar));
            return true;
        }
        return false;
    }

    /**
     * Method to replace characters in the last paragraph
     *
     * @param oldChar the old character.
     * @param newChar the new character.
     * @return returns true if the paragraph is changed successfully
     */
    public boolean replace(String oldChar, String newChar) {
        text.set((text.size() - 1), text.get(text.size() - 1).replace(oldChar, newChar));
        return true;
    }


    public boolean del(int paragraphNumber) {
        if (paragraphExists(paragraphNumber)) {
            text.remove(paragraphNumber - 1);
            return true;
        }
        return false;
    }

    public boolean del() {
        text.remove(text.size() - 1);
        return true;
    }

    public ArrayList<String> index() {
        wordbook = createWordlist();
        ArrayList<String> output = new ArrayList<>();
        for (Map.Entry<String, ArrayList<Integer>> entry : wordbook.entrySet()) {
            String key = entry.getKey();
            ArrayList<Integer> values = entry.getValue();
            if (values.get(0) >= 3) {
                output.add(key);
                for (int value : values) {
                    output.add(String.valueOf(value));
                }
            }
        }
        return output;
    }

    private HashMap<String, ArrayList<Integer>> createWordlist() {
        for (int i = 0; i < text.size(); i++) {
            String[] woerter = text.get(i).trim().toLowerCase().split("[ :;.,!?><\'/]+");
            for (String wort : woerter) {
                ArrayList<Integer> zahlenListe;
                int zähler = 1;
                if (wordbook.containsKey(wort)) {
                    zahlenListe = wordbook.get(wort);
                    zähler = zahlenListe.get(0);
                    zahlenListe.remove(0);
                    zahlenListe.add(0, zähler + 1);

                    if (zahlenListe.get(zahlenListe.size() - 1) < i + 1) {
                        zahlenListe.add(i + 1);
                    }
                    wordbook.put(wort, zahlenListe);
                } else {
                    zahlenListe = new ArrayList<>();
                    zahlenListe.add(1);
                    zahlenListe.add(zähler);
                    wordbook.put(wort, zahlenListe);
                }
            }
        }
        return wordbook;
    }

}
